import socket
import json 
try:
	import freefang.models as models
	import freefang.utils as utils
	import freefang.net as net
	import freefang.packets as packets
	import freefang.roles as roles

except ImportError:
	import models
	import utils
	import net
	import packets
	import roles


import uuid
import threading
import select
import time
import struct 
import argparse
import ssl
import os

VERSIONSTR = """
FreeFang Server 0.2.1
Written by @Solirs and @lukakralik
Licensed under AGPLv3.0
Source code at https://github.com/FreeFangGame/freefang-server
"""

def parse_ruleset(ruleset, game):
	if ruleset.town_voting_scheme:
		game.town_voting_scheme = ruleset.town_voting_scheme.strip()
		delattr(ruleset, "town_voting_scheme")
	if ruleset.werewolf_voting_scheme:
		game.werewolf_voting_scheme = ruleset.werewolf_voting_scheme.strip()
		delattr(ruleset, "werewolf_voting_scheme")
	
		
	ret = {}
	for key, value in ruleset.__dict__.items():
		ret[getattr(roles, key)] = value
	game.roles = ret
	

def game_creation_loop(args):
	
	games = {}
	# Create game object 

		
	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
	s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1) #For devs only
	s.setblocking(0)
	s.bind((args.addr, args.port))
	s.listen()
	if args.cert and args.key:
		print("SSL is ENABLED.")
		context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
		context.load_cert_chain(args.cert, args.key)
		s = context.wrap_socket(s, server_side=True)
	print(f"Listening on {args.addr}:{args.port}")
	
	inputs = [s]
	outputs = []
	
	connections = {} # Associating connections to players
	
	
	while True:
		time.sleep(0.05)
		read, write, excp = select.select(inputs, outputs, inputs)
		
		for i in read:
			if i is s:
				print("New connection")
				try:
					con, addr = s.accept() # Accept new connection
					con.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER, struct.pack('ii', 1, 0))
					inputs.append(con) 
					outputs.append(con)
					connections[con] = 0
					con.setblocking(0)
					
				except ssl.SSLError: 
					# SSL related errors may be thrown but they should not be fatal.
					pass
					 

			else:
				try:
					packet = net.read_packet(i)
					if not packet: # If no packet is sent we bail out
						net.send_packet("", i)
						continue
					packet = utils.json_to_object(packet)
					
					# Game creation
					if packet.action == "game_create":
						gameid = str(uuid.uuid4())
						game = models.WWgame()
						game.playercap = packet.headers.playercap
						parse_ruleset(packet.headers.ruleset, game)
	
						for rl in game.roles: 
							if rl.nightrole:
								game.nightroles.append(rl)
							
						games[gameid] = game
						net.send_packet(utils.obj_to_json(packets.Game_created(gameid=gameid)), i) # Send player a packet confirming success
						
						print("New game")
					
					# Game joining
					elif packet.action == "game_join":
						if not games.get(packet.headers.gameid): # If the game for the proposed gameid doesn't exist
							net.send_packet(utils.obj_to_json(packets.Action_failure(error="game_not_found")), i)
						else:
							game = games[packet.headers.gameid]
								
							p = models.Player()
							p.name = packet.headers.name
							p.connection = i
							connections[i] = p
							p.game = game
							for spl in game.players:
								net.send_packet(utils.obj_to_json(packets.Player_join(username=p.name)), spl.connection)
							game.players.append(p)
							net.send_packet(utils.obj_to_json(packets.Added_to_game(username=packet.headers.name, players=[i.name for i in game.players])), i) # Send player a packet confirming success

							if len(game.players) == game.playercap:

								thread = threading.Thread(target=game.gameloop)
								for i in game.players:
									inputs.remove(i.connection)
									outputs.remove(i.connection)
								del games[packet.headers.gameid]
								thread.start()
					elif packet.action == "pre_game_message":
						sender = connections[i]
						for spl in sender.game.players:
							net.send_message(sender.name, packet.headers.message, spl.connection)
								
				except Exception as e:
					print(e)
					inputs.remove(i)
					outputs.remove(i)
					player = connections[i]
					if player:
						player.game.players.remove(player)
						for spl in player.game.players:
							net.send_packet(utils.obj_to_json(packets.Player_leave(username=player.name)), spl.connection)
					del connections[i]
						
				
def main():
	parser = argparse.ArgumentParser()
	parser.add_argument("-v", "--version", help="Show version", action="store_true")

	parser.add_argument("-p", "--port", help="The port for the server to listen on (default: 9999)", type=int, default="9999")
	parser.add_argument("-i", "--addr", help="The address to listen on (default: 0.0.0.0)", type=str, default="0.0.0.0")

	parser.add_argument("-c", "--cert", help="SSL certificate if you choose to use SSL.", type=os.path.abspath, default=None)
	parser.add_argument("-k", "--key", help="SSL key if you choose to use SSL.", type=os.path.abspath, default=None)
	args = parser.parse_args()
	
	if args.version:
		print(VERSIONSTR)
		return

	print("Starting FreeFang server.")


	game_creation_loop(args)
	
	
	
if __name__ == "__main__":
	main()



import struct

try:
	import freefang.utils as utils
	import freefang.packets as packets
	
except ImportError:
	import packets
	import utils


def readlength(con): # Read the length of the packet prepended to it

	try:
		unpack = con.recv(4) 
		leng = struct.unpack("<I", unpack)[0]
		return leng
	except:
		return None
			
def read_packet(con):
	length = readlength(con)
	if not length:
		return None
	return con.recv(length).decode()

def send_packet(packet, con):
	
	con.sendall(struct.pack("<I", len(packet)) + packet.encode()) 
		
# This function sends a success packet over a connection, this is just a cleaner way of doing it so you dont have to
# copy that long line all over the codebase
def send_success(con):
	send_packet(utils.obj_to_json(packets.Action_success()), con)
	
def send_failure(con, error):
	send_packet(utils.obj_to_json(packets.Action_failure(error=error)), con)

def send_message(sender, message, con):
    timestamp = utils.get_current_timestamp()
    chat_message = packets.ChatMessage(sender, message, timestamp)
    send_packet(utils.obj_to_json(chat_message), con)
    



class Added_to_game: #Packet for when a player is added to a game
	def __init__(self, username, players):
		self.action = "added_to_game"
		self.headers = {
			"username": username,
			"players": players # List of the players currently in the game
		}

class Player_join:
	def __init__(self, username):
		self.action = "player_join"
		self.headers = {
			"name": username,
		}
class Player_leave:
	def __init__(self, username):
		self.action = "player_leave"
		self.headers = {
			"name": username,
		}

class Game_created:
	def __init__(self, gameid):
		self.action = "game_created"
		self.headers = {
			"id": gameid
		}


class Role_attributed:
	def __init__(self, role):
		self.action = "role_attributed"
		self.headers = {
			"role": role
		}

class Role_wakeup:
	def __init__(self, role):
		self.action = "role_wakeup"
		self.headers = {
			"role": role
		}

class Time_change:
	def __init__(self, time):
		self.action = "time_change"
		self.headers = {
			"time": time,
		}
# Event for when a werewolf votes, should only be sent to werewolves
class Werewolf_vote:
	def __init__(self, target, sender):
		self.action = "werewolf_vote"
		self.headers = {
			"sender": sender,
			"target": target
		}

# Event for when someone votes during the day
class Town_vote:
	def __init__(self, target, sender):
		self.action = "town_vote"
		self.headers = {
			"sender": sender,
			"target": target
		}

class Show_werewolves: # This packet is sent to all werewolves in the beginning of a game to know who other werewolves are
	def __init__(self, werewolves):
		self.action = "show_werewolves"
		self.headers = {
			"werewolves": werewolves,
		}

# This packet should be sent back to a player after something is done successfully.
# For example if a werewolf sends a vote packet and it is taken into account, then return this packet
class Action_success:
	def __init__(self):
		self.action = "action_success"

# On the flip side this packet should be sent if there was an error processing a packet (i.e werewolf vote sent during the day)
class Action_failure:
	def __init__(self, error):
		self.action = "action_failure"
		self.headers = {
			"error": error
		}
		
class Player_death:
	def __init__(self, name, role, reason):
		self.action = "player_death"
		self.headers = {
			"name": name,
			"role": role,
			"reason": reason
		}
		
class Game_end:
	def __init__(self, outcome):
		self.action = "game_end"
		self.headers = {
			"outcome": outcome
		}

class ChatMessage:
    def __init__(self, sender, message, timestamp):
        self.action = "chat_message"
        self.headers = {
            "sender": sender,
            "message": message,
            "timestamp": timestamp
        }

class SeerReveal:
    def __init__(self, role, name):
        self.action = "seer_role_reveal"
        self.headers = {
			"role": role,
			"name": name
        }
        
class Town_Vote_Begin:
	def __init__(self):
		self.action = "town_vote_begin"

class Check_alive:
	def __init__(self):
		self.action = "check_alive"
	




try:
    import freefang.net as fn
    import freefang.packets as packets
    import freefang.utils as utils
    import freefang.voting as voting
except ImportError:


    import net as fn
    import packets
    import utils
    import voting

import json
from operator import attrgetter


class Role:
	def __init__(self):
		self.number = 2 # Number of people with this role


# There should be more roles in the future that will inherit those
class Villager(Role):
	nightrole = 0
	def __init__(self):
		super(Villager, self).__init__()
		
		pass
	@staticmethod
	def vote(headers, game, connection):
		
		# Get the player object of the target
		target = game.getplayerbyname(headers.target)


		# Check if time is day and sender hasnt voted yet
		# Reminder that game.up = 0 means that its day
		if game.up == 0 and headers.sender.voted == False and headers.sender.alive and target.alive:
			# Create vote object
			vt = TownVote(target, headers.sender)
			
			
			
			# Cast vote
			game.votes.append(vt)
			
			# Mark sender as already voted
			headers.sender.voted = True
			

			event = packets.Town_vote(target=headers.target, sender=headers.sender.name) # Create a packet indicating a person was voted
			pckt = utils.obj_to_json(event) # Serialize it to json
			game.sendall(pckt) # Send it to everyone
			
			if len(game.votes) == len(game.alive): # Everyone voted
				if game.town_voting_scheme == "relmaj":
					voting.relmaj(game)
				elif game.town_voting_scheme == "absmaj":
					voting.absmaj(game)
				return 2

				
			
			

			

class Werewolf(Role):
	nightrole = 1
	order = 3
	def __init__(self):
		super(Werewolf, self).__init__()
		pass
		
	@staticmethod
	def vote(headers, game, connection):
		
		target = game.getplayerbyname(headers.target)

		if game.up == Werewolf and headers.sender.iswerewolf() and headers.sender.voted == False and target.alive and headers.sender.alive:
			if target == headers.sender.protected:
				return 3  # protected player case (3)
			vt = WerewolfVote(target, headers.sender)
			game.votes.append(vt)
			headers.sender.voted = True
			event = packets.Werewolf_vote(target=headers.target, sender=headers.sender.name) # Create a packet indicating a person was voted
			pckt = utils.obj_to_json(event) # Serialize it to json
			game.sendwerewolves(pckt) # Send it to all werewolves

			if len(game.votes) == len(game.werewolves): # All the werewolves voted
				if game.werewolf_voting_scheme == "unanimity":
					voting.unanimity(game)
				elif game.werewolf_voting_scheme == "relmaj":
					voting.relmaj(game)
				elif game.werewolf_voting_scheme == "absmaj":
					voting.absmaj(game)
				for i in game.werewolves:
					i.voted = False
				return 2

					
				 
			return 0
		else:
			return 1
		
		
# The seer is supposed to be able to learn about the role of one player each night
class Seer:
	nightrole = 1
	order = 2
	@staticmethod
	def reveal(headers, game, connection):
		target = game.getplayerbyname(headers.target)
		# Check if the player is actually a seer, if the role currently woken up is seer, and if the target is alive
		if game.up == Seer and headers.sender.role == Seer and target.alive and headers.sender.alive:
			# Create packet
			packet = packets.SeerReveal(target.role.__name__, target.name)
			# Send packet containing the desired info to the seer
			game.send_packet(utils.obj_to_json(packet), connection)
			return 2
		return 1
		
# The hunter gets to kill a player of his choice upon death
class Hunter:
	nightrole = 0
	@staticmethod
	def kill(headers, game, connection):
		target = game.getplayerbyname(headers.target)
		if game.up == Hunter and headers.sender.role == Hunter and target.alive:
			game.kill_player(target, reason="hunter_kill")
			return 2
		return 1

# can prevent one person of his choosing from dying at each night	
class Protector(Role):
	nightrole = 1
	order = 1
	def __init__(self):
		super(Protector, self).__init__()
	
	@staticmethod
	def protect(headers, game, connection):
		target = game.getplayerbyname(headers.target)
    
		# if both protector and target are alive, modify the headers
		if headers.sender.alive and target.alive and game.up == Protector:
			target.protected = True
			return 2
		return 1

class Vote:
    def __init__(self, target, sender):
        self.sender = sender
        self.target = target

class TownVote(Vote):
    def __init__(self, target, sender):
        super(TownVote, self).__init__(target, sender)
        pass

class WerewolfVote(Vote):
    def __init__(self, target, sender):
        super(WerewolfVote, self).__init__(target, sender)
        pass



"""
This file contains the code for the different voting schemes one can choose for each side
The schemes are absolute majority, relative majority, and unanimity.
"""

def unanimity(game):
	unanimity = all(i.target == game.votes[0].target for i in game.votes) 
	if not unanimity: # Check if everyone voted unanimously
		# No kill if unanimity is not achieved
		return 1
	else:
		# Kill the player that was unanimously voted
		kill = game.votes[0].target
		game.kill_player(kill)
		# Reset game.votes
		game.votes = []
		for i in game.players:
			i.voted = False # Set all players like they havent voted yet
		return 0

def relmaj(game):
	votes = {}
				
	# Gather all votes in a dictionary along with their target
	for i in game.votes:
		votes.setdefault(i.target, 0)
		votes[i.target] += 1
				
	# Get the highest number of votes on a player
	maxvotes = max(votes.values())
				
	# Check if that player is tied with another for the amount of votes he was targeted by
	ties = [key for key, value in votes.items() if value == maxvotes]
		

	for i in game.players:
		i.voted = False # Set all players like they havent voted yet
	game.votes = []
		
	# If there is a tie, no death (this is temporary, there may be vote altering roles in the future)
	if len(ties) > 1:
		return 1
				
	game.kill_player(ties[0])

def absmaj(game):
	votes = {}
	
	for i in game.votes:
		votes.setdefault(i.target, 0)
		votes[i.target] += 1
	
	for player, votes in votes.items():
		if votes > (len(game.alive)/2):
			game.kill_player(player)
			for i in game.players:
				i.voted = False # Set all players like they havent voted yet
			game.votes = []
				
			return 0
	for i in game.players:
		i.voted = False # Set all players like they havent voted yet
		game.votes = []
				
	return 1
			
		
		



import socket
import time
json = """
{
	"action": "werewolf_vote",
	"headers": {
		"target": "jyunlzrkcm"
	}
}
"""
#json = str(len(json)) + "\r" + json
print(json)
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
s.connect(("127.0.0.1", 9999))
while True:
	i = s.recv(4096).decode()
	if i:
		print(i)
		s.send((str(len(json)) + "\r").encode())
		s.send(json.encode())
		s.recv(4096)
#	s.recv(1024)



import random, string, json
from types import SimpleNamespace
import datetime

def randstring():
	alphabet = list("abcdefghijklmnopqrstuvwxyz")
	return "".join([alphabet[random.randint(0, 25)] for i in range(10)])

def json_to_object(data): # Turn json data into an object
	return json.loads(data, object_hook=lambda d: SimpleNamespace(**d))

def obj_to_json(obj):
	return json.dumps(obj.__dict__)

def get_current_timestamp():
    current_time = datetime.datetime.now()
    timestamp = current_time.strftime("%Y-%m-%d %H:%M:%S")
    return timestamp



import select, random, json, traceback, sys, datetime, time
try:
	from freefang.roles import *
	import freefang.net as fn
	import freefang.packets as packets
	import freefang.utils as utils


except ImportError:
	from roles import *
	import net as fn
	import packets
	import utils


class Player:
	def __init__(self):
		self.role = None
		self.alive = True
		self.name = ""
		self.connection = 0
		self.voted_by = 0 # Number of people who voted for this player
		self.voted = False
		self.time = 0 # 0 = Night, 1 = Day
		self.protected = None # name of the protected player
		self.game = None
	def iswerewolf(self):
		return issubclass(self.role, Werewolf) and self.alive
			


class WWgame:
	def __init__(self):
		self.nightdeaths = []
		self.players = []
		self.alive = []
		self.dead = []
		self.playercap = 0
		self.werewolves = []
		self.villagers = []
		self.socket = 0
		self.inputs = []
		self.outputs = []
		self.msgqueues = {}
		self.nightroles = [] # Roles that should be woken up at night, in order
		self.up = 0 # The current role which is woken up, 0 if day.
		self.action_to_function = {"werewolf_vote": Werewolf.vote, "town_vote": Villager.vote, "town_message": self.townmessage, "werewolf_message": self.werewolfmessage, "hunter_kill": Hunter.kill, "seer_reveal":Seer.reveal, "protector_protect": Protector.protect}
		self.votes = []
		self.connections = {} # Dictionnary associating connections to players

		self.roles = {} # The number of players for each role should be decided by the client upon game creation and should be implemented alongside the protocol
		
		self.town_voting_scheme = "absmaj"
		self.werewolf_voting_scheme = "relmaj"
	
	# This function is a shortcut to send a packet and remove the player if the connection is dead
	def send_packet(self, packet, con):
		try:
			fn.send_packet(packet, con)
		except (BrokenPipeError, ConnectionResetError):
			self.remove_player(con)
		
	def distribute_roles(self):
	# Get all the players and keep track of those with no roles
		print("Distributing roles to players")
		
		# Randomly give each role to the number of players its supposed to be on, 
		for spl in self.players:
			playerrole, _ = random.choice(list(self.roles.items()))
			self.roles[playerrole] -= 1
			if self.roles[playerrole] == 0:
				del self.roles[playerrole]
			
			spl.role = playerrole
			fn.send_packet(utils.obj_to_json(packets.Role_attributed(role=playerrole.__name__)), spl.connection)
			print(f"Player {spl.name} got role {playerrole.__name__}") 
			
			if spl.iswerewolf():
				self.werewolves.append(spl)
			else:
				self.villagers.append(spl)
			
				
	def townmessage(self, headers, game, connection):

		for i in self.players:
			fn.send_message(headers.sender.name, headers.message, i.connection) # Send it to all players
	
	def werewolfmessage(self, headers, game, connection):

		for i in self.werewolves:
			fn.send_message(headers.sender.name, headers.message, i.connection) # Send it to all werewolves
		
		
			
	def update_player_count(self):
		num_players = len(self.players)
		print(f"Number of present players: {num_players}")
	
	def remove_player(self, i):
		self.inputs.remove(i)
		self.outputs.remove(i)
		player = self.connections[i]
		self.players.remove(player)
		if player in self.villagers:
			self.villagers.remove(player)
		elif player in self.werewolves:
			self.werewolves.remove(player)
		for spl in self.players:
			fn.send_packet(utils.obj_to_json(packets.Player_leave(username=player.name)), spl.connection)

						
		print(f"{player.name} disconnected")
		
	
	# This function checks if the conditions for game end are met
	def game_continues(self):
		return (len(self.werewolves) < len(self.villagers) and len(self.werewolves) > 0)
		
		
	def kill_player(self, player, reason=None):
		
		# If the player is protected we do nothing. 
		if player.protected:
			return 1
		print(player.name + " died")
		player.alive = False

		
		# If the player died during the night we keep it in a list to notify the other players when the day rises
		# Otherwise we just send the packet right away
		if self.up != 0 and self.up != Hunter:
			self.nightdeaths.append(player)
			return 0
			
		
		# Notify everyone that player died
		event = packets.Player_death(name=player.name, role=player.role.__name__, reason=reason)
		
		self.sendall(utils.obj_to_json(event))
		
		# Remove player from various lists reserved to living players
		if player in self.werewolves:
			self.werewolves.remove(player)
		else:
			self.villagers.remove(player)
			
		self.alive.remove(player)
		
		# If the dead player is a hunter we need to wait for him to make his kill
		if player.role == Hunter:
			self.up = Hunter
			self.eventloop()
		
		# Add player to list of dead players
		self.dead.append(player)
		

	def handle_disconnections(self):
		for player in self.players:
			try:
				fn.send_packet("", player.connection)  # Sending empty message to check connection status
			except:
				self.remove_player(player.connection)
	


		
	def isnight(self):
		return self.time == 0
	# Those two functions queue a message to be sent to all players or all wolves during the select loop
	def queueall(self, string): # Send a message to all players
		for i in self.outputs:
			self.msgqueues.setdefault(i, [])
			self.msgqueues[i].append(string)

	def queuewerewolves(self, string): # Send a message to all wolves
		for i in self.werewolves:				
			self.msgqueues.setdefault(i.connection, [])
			self.msgqueues[i.connection].append(string)
				
	# Those two functions instantly send a packet to their destined targets, either all players or all werewolves
	def sendall(self, string):
		for i in self.players:
			self.send_packet(string, i.connection)
			
	def sendwerewolves(self, string):
		for i in self.werewolves:
			self.send_packet(string,i.connection)
			

	def getplayerbyname(self, name):
		return [i for i in self.players if i.name == name][0]
		
	def eventloop(self): 
		end = None
		while not end: # This loop will eventually be broken, can be while true.
			time.sleep(0.05)

			read, write, exceptional = select.select(self.inputs, self.outputs, self.inputs)
			for i in read: # Check every connection that has sent a message
				try:
						
					packet = fn.read_packet(i)
					if not packet:
						fn.send_packet("", i)

						continue

					try:
						pckt = utils.json_to_object(packet) # Create an object from the packet
						print(pckt.action)
						setattr(pckt.headers, "sender", self.connections[i]) # Add the packet's sender to the object to use it in the event function
						ret = self.action_to_function[pckt.action](pckt.headers, self, i) # Call an event function depending on the packet's action
						if not ret: 
							fn.send_success(i)
						elif ret == 2: # End our loop and go to the next role
							fn.send_success(i)
							
							# Set end to true to end the loop after message sending is done, then break as no more packets are needed
							end = True
							break
						else: # If 1 is returned then something went wrong, the packet is probably bad (for example voting someone who doesnt exist)
							fn.send_failure(i, error=None)
							pass
					except Exception as e:
						traceback.print_exc()
						fn.send_failure(i, error=None)
				except:
					self.remove_player(i)
					


			for i in write:
				if self.msgqueues.get(i): # If a message is pending for a player send it to them
					for x in self.msgqueues[i]:
						self.send_packet(x, i)
					del self.msgqueues[i] # No more message needed to send
					
				else:
					continue
			
			for i in exceptional:  
				self.remove_player(i)
	
		
	def gameloop(self):
		# Start game and distribute roles
		print("Game starting")
		
		# Sort night roles by the order at which they should wake up (see roles.py)
		self.nightroles.sort(key=lambda x: x.order)
		
		self.handle_disconnections()
		self.distribute_roles()
		
		# Create the list of alive players
		self.alive = [i for i in self.players]
		
		for i in self.players:
			self.connections[i.connection] = i
			
		# Send all werewolves the list of werewolves in the game
		werewolvenames = [i.name for i in self.werewolves]
		self.sendwerewolves(utils.obj_to_json(packets.Show_werewolves(werewolves=werewolvenames)))
		

		# Setup I/O channels for select as well as message queue for each player
		self.inputs = [i.connection for i in self.players]
		self.outputs = [i.connection for i in self.players]

		while self.game_continues(): 
			# Game should go on as long as there are villagers and werewolves, keeping the day night cycle
			self.sendall(utils.obj_to_json(packets.Time_change(time="night"))) # Notify everyone night has fallen
			for i in self.nightroles:
				self.queueall(utils.obj_to_json(packets.Role_wakeup(role=i.__name__))) # Notify everyone role has woken up
				self.up = i
				self.eventloop()
			
			# Remove all protections 
			for player in self.players:
				player.protected = None
			self.sendall(utils.obj_to_json(packets.Time_change(time="day"))) # Notify everyone day has risen
			self.up = 0
			self.time = 1


			
			
			# Notify all players of deaths that happened during the night. 
			for death in self.nightdeaths:
				self.kill_player(death)
			self.nightdeaths = []
			self.up = 0
			if not self.game_continues(): # Game ended during the night, we dip
				break
			self.sendall(utils.obj_to_json(packets.Town_Vote_Begin()))



			self.eventloop()
			
		# The game ends if the loop above is over
		
		print("Game end")
		
		outcome = ""
		# If all werewolves are dead aka villager win, otherwise its a werewolf win
		if len(self.werewolves) == 0:
			outcome = "town_win"
		else:
			outcome = "werewolf_win"
			
		self.sendall(utils.obj_to_json(packets.Game_end(outcome=outcome)))
		
		
		





from setuptools import setup, find_packages

setup(
    name='FreeFang',
    version='0.2.1',
    author='FreeFang Development Team',
    description='A libre implementation of the Werewolf game, also known as Mafia.',
    packages=find_packages(),
	entry_points={
		'console_scripts': [
			'freefang-server = freefang.main:main',
		],
	},

)

